<?php

declare(strict_types=1);

namespace FiloBlu\GcpLogging\Observer;

use Exception;
use FiloBlu\GcpLogging\Helper\Data;
use Google\Cloud\Logging\LoggingClient;
use Magento\Backend\Model\Auth\Session;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Message\ManagerInterface;

/**
 *
 */
class CheckConfigData implements ObserverInterface
{
    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Session
     */
    protected $authSession;

    /**
     * @var ManagerInterface
     */
    protected $messageManager;

    /**
     * @param \FiloBlu\GcpLogging\Helper\Data $helper
     * @param \Magento\Backend\Model\Auth\Session $authSession
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     */
    public function __construct(
        Data $helper,
        Session $authSession,
        ManagerInterface $messageManager
    ) {
        $this->helper = $helper;
        $this->authSession = $authSession;
        $this->messageManager = $messageManager;
    }

    /**
     * Observer to check the gcp configuration set to BE.
     * It works for Magento 2.3 and higher, due to lack of event dispatch on the lower versions.
     *
     * @param Observer $observer
     */
    public function execute(Observer $observer)
    {
        $section = $observer->getDataByPath('configData/section');

        if ($section !== 'filoblu_gcp_logging') {
            return;
        }

        $currentUser = $this->authSession->getUser();

        if (!$currentUser) {
            return;
        }

        $type = $this->helper->getGcpLoggingType();

        if ($type !== Data::LOG_TYPE_GCP) {
            return;
        }

        try {
            $client = new LoggingClient($this->helper->getLoggingClientConfiguration());
            $client->psrLogger($this->helper->getLogFileName(), ['labels' => ['context' => 'Filoblu_Gcp_Logging']])
                ->log('INFO', 'Save Gcp Logging Config', ['user' => $currentUser->getUserName()]);
        } catch (Exception $e) {
            $this->messageManager->addWarningMessage('GCP configuration error: ' . $e->getMessage());
        }
    }
}
